import { EventEmitter, ElementRef, Renderer2, ViewChild, Output, Input, Component, forwardRef, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR, NG_VALIDATORS, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { MultiLingualData } from '@rxweb/localization';
import { CommonModule } from '@angular/common';

class TagModel {
    constructor(id, text, active, isRecordInvactive = false) {
        this.id = id;
        this.text = text;
        this.active = active;
        this.isRecordInvactive = isRecordInvactive;
    }
}

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var RxTagComponent_1;
let RxTagComponent = RxTagComponent_1 = class RxTagComponent {
    constructor(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.propagateChange = () => { };
        this.dropIndex = 0;
        this.inputDisabled = false;
        this.isLimitExceeded = false;
        this.isInValid = false;
        this.isCalledFromWriteValue = false;
        this.isSpinner = false;
        this.isServerCalled = false;
        this.removed = new EventEmitter();
        this.added = new EventEmitter();
        this.focus = false;
        this.mainSource = new Array();
        this.selectedTags = new Array();
        this.dropSource = new Array();
        this.inActiveSource = new Array();
        this.maxSelectionMessage = MultiLingualData.get("global.control.rx-tag.message-max-selection");
        this.splitCharacter = MultiLingualData.get("global.control.rx-tag.split-character");
        let sortConfiguration = MultiLingualData.get("global.control.rx-tag.is-sort");
        this.isSort = (sortConfiguration === '') ? this.isSort : sortConfiguration;
        this.placeholder = this.placeholder || MultiLingualData.get("global.control.rx-tag.placeholder");
        this.placeholder = this.placeholder || "";
        if (!this.splitCharacter)
            this.splitCharacter = ",";
        this.element = elementRef.nativeElement;
    }
    set source(value) {
        if (value) {
            this.userSource = value;
            if (this.keyValueProps)
                this.setMainSource(this.userSource);
        }
    }
    ngAfterContentInit() {
        if (this.formControl)
            this.writeValue(this.formControl.value, true);
        this.checkValid(true);
        if (this.userSource && this.mainSource.length === 0) {
            this.setMainSource(this.userSource);
            if (this.currentValue !== undefined && this.currentValue !== "")
                this.setSelectedTags(this.mainSource, this.currentValue);
        }
        else if (this.itemTexts && (this.currentValue !== undefined && this.currentValue !== "")) {
            var selectedItemSource = [];
            var splitIds = String(this.currentValue).split(this.splitCharacter);
            this.itemTexts.split(this.splitCharacter).forEach((t, i) => {
                var jObject = {};
                jObject[this.keyValueProps[1]] = splitIds[i];
                var textPattern = this.keyValueProps[0].split(/#/g);
                jObject[textPattern[0]] = t;
                selectedItemSource.push(jObject);
            });
            var itemSource = this.getTagModelSource(selectedItemSource);
            if (this.currentValue)
                this.setSelectedTags(itemSource, this.currentValue);
        }
    }
    ngOnChanges(source) {
        if (this.focus)
            var t = setTimeout(() => {
                this.applyFocus(this.viewChild.nativeElement);
            }, 1000);
    }
    checkValid(isValid) {
        if (isValid)
            this.validTimeOutId = window.setTimeout(() => {
                for (var i = 0; i < this.element.classList.length; i++) {
                    this.isInValid = this.element.classList[i] == "ng-invalid";
                    if (this.isInValid)
                        break;
                }
                window.clearTimeout(this.validTimeOutId);
                this.validTimeOutId = undefined;
            }, 500);
        else
            this.isInValid = isValid;
    }
    setMainSource(source) {
        this.inActiveSource = new Array();
        this.mainSource = this.getTagModelSource(source);
        this.mainSource.forEach(t => this.dropSource.push(t));
        this.sortSource();
    }
    sortSource() {
        this.dropSource = this.isSort === undefined || this.isSort === true || String(this.isSort) == "true" ? this.dropSource.sort(this.customSort((t) => t.text, false)) : this.dropSource;
    }
    customSort(predicate, orderByDescending) {
        return (a, b) => {
            const first = predicate(a);
            const second = predicate(b);
            if (first > second) {
                return !orderByDescending ? 1 : -1;
            }
            else if (first < second) {
                return !orderByDescending ? -1 : 1;
            }
            else {
                return 0;
            }
        };
    }
    getTagModelSource(source) {
        var textPattern = this.keyValueProps[0].split(/#/g);
        var tags = new Array();
        source.forEach(t => {
            var text = '';
            textPattern.forEach(x => {
                if (x != "")
                    if (t[x] != undefined)
                        text += t[x];
                    else
                        text += x;
            });
            if ((t["status"] == undefined) || (t["status"] != undefined && t["status"] == true))
                tags.push(new TagModel(t[this.keyValueProps[1]], text, false));
            else
                this.inActiveSource.push(new TagModel(t[this.keyValueProps[1]], text, false, true));
        });
        return tags;
    }
    setSelectedTags(source, tags) {
        this.selectedTags = new Array();
        if (tags)
            String(tags).split(this.splitCharacter).forEach(v => {
                if (v != "") {
                    var baseId = v;
                    var result = source.filter(t => t.id == baseId);
                    if (result.length > 0 && this.selectedTags.filter(t => t.id == baseId).length === 0)
                        this.selectTag(result[0]);
                    else {
                        result = this.inActiveSource.filter(t => t.id == baseId);
                        if (result.length > 0 && this.selectedTags.filter(t => t.id == baseId).length === 0)
                            this.selectTag(result[0]);
                    }
                }
            });
    }
    setDropSource(source) {
        this.dropSource = new Array();
        source.forEach(t => this.dropSource.push(t));
    }
    applyFocus(tagSearch) {
        tagSearch.focus();
        this.focus = false;
    }
    reset() {
        this.setDropSource(this.mainSource);
        this.setSelectedTags(this.mainSource, "");
        this.propagateChange(undefined);
    }
    changeTagSource(source) {
        this.inActiveSource = new Array();
        var tagModels = this.getTagModelSource(source);
        this.setDropSource(tagModels);
        this.setSelectedTags(tagModels, this.getTagString());
    }
    showTag(tags, clearExisting) {
        if (tags && !this.limitExceded()) {
            if (clearExisting && tags.split(this.splitCharacter).length > parseInt(this.maxSelection)) {
                return;
            }
            this.selectedTags.forEach(tag => this.dropSource.push(tag));
            tags = clearExisting ? tags : this.getTagString() + this.splitCharacter + tags;
            this.setSelectedTags(this.dropSource, tags);
        }
    }
    clear() {
        this.isCalledFromWriteValue = false;
        this.selectedTags.forEach(t => this.dropSource.push(t));
        this.sortSource();
        this.selectedTags = new Array();
        this.updateFormValue();
    }
    selectAll() {
        var dropTags = new Array();
        this.dropSource.forEach(t => dropTags.push(t));
        dropTags.forEach(t => this.selectTag(t));
    }
    getTagString() {
        let tagString = '';
        var selectCount = 0;
        this.selectedTags.forEach(t => {
            if ((selectCount === this.selectedTags.length - 1)) {
                if (t.id == -1)
                    tagString += t.text;
                else
                    tagString += t.id;
            }
            else {
                if (t.id == -1)
                    tagString += t.text + this.splitCharacter;
                else
                    tagString += t.id + this.splitCharacter;
            }
            selectCount++;
        });
        return tagString;
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched() { }
    writeValue(value, fromNg = false) {
        if (value !== undefined)
            this.currentValue = value;
        if (this.selectedTags.length > 0) {
            this.isCalledFromWriteValue = true;
            this.ngAfterContentInit();
        }
        if (this.formControl && !fromNg)
            this.formControl.setValue(this.currentValue);
    }
    removeTag(tag) {
        if (!this.tagDisabled) {
            this.removed.emit(tag);
            var indexOf = this.selectedTags.indexOf(tag);
            this.selectedTags.splice(indexOf, 1);
            if (!tag.isRecordInvactive && tag.id != -1)
                if (this.isServerCalled || !this.lookup) {
                    this.dropSource.push(tag);
                    this.sortSource();
                }
            this.updateFormValue();
        }
    }
    onFocus(event) {
        if (!this.currentValue)
            this.dropSource = this.mainSource;
        this.setActiveTabClass();
    }
    onBlur(event) {
        if (this.timeOutId)
            window.clearTimeout(this.timeOutId);
        this.timeOutId = window.setTimeout(() => {
            if (this.freeTextEnabled) {
                this.enter(event.target);
            }
            this.clearActiveTabClass();
        }, 200);
    }
    removeDoubleBackslashes(input) {
        let separator = ['\\'];
        let result = "";
        let subResult = input.split(separator);
        for (var i = 0; i <= subResult.length - 1; i++) {
            result = i < subResult.length - 1 ? result + subResult[i] + "\\" : result + subResult[i];
        }
        return result;
    }
    onKeyup(event, scroller) {
        var value = event.srcElement.value;
        var regex = new RegExp(this.removeDoubleBackslashes(value.toLocaleLowerCase()));
        if (value !== null && event.keyCode !== 40 && event.keyCode !== 13 && event.keyCode !== 37) {
            this.dropSource = this.mainSource.filter(t => regex.test(t.text.toLowerCase()));
            this.selectedTags.forEach(t => {
                if (this.dropSource.length > 0) {
                    var indexOf = this.dropSource.indexOf(t);
                    if (indexOf != -1)
                        this.dropSource.splice(indexOf, 1);
                }
            });
            this.setActiveTabClass();
        }
    }
    onKeydown(event, scroller) {
        if (event.keyCode === 40) {
            event.preventDefault();
            if (this.dropSource.length !== this.dropIndex) {
                if (this.dropIndex !== 0)
                    this.dropSource[this.dropIndex - 1].active = false;
                this.dropSource[this.dropIndex].active = true;
                (this.dropIndex !== this.dropSource.length - 1) ? this.dropIndex++ : 0;
                this.changeScroll(scroller);
                if (this.openDrop === "")
                    this.setActiveTabClass();
            }
        }
        else if (event.keyCode === 38) {
            event.preventDefault();
            this.dropSource[this.dropIndex].active = false;
            if (this.dropIndex !== 0)
                this.dropIndex--;
            this.dropSource[this.dropIndex].active = true;
            (this.dropIndex == 0) ? this.dropIndex++ : 0;
        }
        else if (event.keyCode === 13) {
            event.preventDefault();
            const selectedTag = this.dropSource.filter(t => t.active);
            if (selectedTag.length > 0) {
                this.selectTag(selectedTag[0]);
                this.clearValue(event.srcElement);
            }
        }
    }
    limitExceded() {
        this.isLimitExceeded = !(parseInt(this.maxSelection) !== this.selectedTags.length);
        return this.isLimitExceeded;
    }
    setActiveTabClass() {
        if (this.limitExceded())
            this.maxSelectionMessage = this.maxSelectionMessage.replace('#maxSelection#', this.maxSelection);
        this.activeTag = "chosen-container-active";
        this.openDrop = "chosen-with-drop";
    }
    changeScroll(scroller) {
        if (this.dropIndex > 8) {
            scroller.scrollTop = scroller.scrollTop - 50;
        }
    }
    enter(targetElement) {
        if (this.freeTextEnabled) {
            var value = targetElement.value;
            if (value) {
                var tagModel = new TagModel(-1, value, true);
                var result = this.selectedTags.filter(t => t.text.toLowerCase() == value.toLowerCase())[0];
                if (!result) {
                    this.selectedTags.push(tagModel);
                    this.updateFormValue();
                }
                this.clearValue(targetElement);
            }
        }
    }
    onClick(tag, tagSearchElement) {
        this.selectTag(tag);
        this.clearValue(tagSearchElement);
    }
    selectTag(tag) {
        this.added.emit(tag);
        tag.active = false;
        var indexOf = this.dropSource.indexOf(tag);
        this.dropSource.splice(indexOf, 1);
        this.selectedTags.push(tag);
        this.clearActiveTabClass();
        if (!this.isCalledFromWriteValue)
            this.updateFormValue();
        this.isCalledFromWriteValue = false;
    }
    updateFormValue() {
        var value = "";
        var count = 0;
        this.selectedTags.forEach(t => {
            if (count === 0) {
                if (t.id == -1)
                    value += t.text;
                else
                    value += t.id;
            }
            else {
                if (t.id == -1)
                    value += this.splitCharacter + t.text;
                else
                    value += this.splitCharacter + t.id;
            }
            count++;
        });
        if (!this.isServerCalled)
            this.currentValue = value;
        this.propagateChange(this.selectedTags.length == 0 ? undefined : value);
        if (this.formControl)
            this.formControl.patchValue(value);
    }
    onMouseover(tag) {
        this.dropSource.forEach(t => t.active = false);
        tag.active = true;
    }
    clearValue(element) {
        element.value = "";
        //this.renderer.invokeElementMethod(element, "focus", []);
    }
    clearActiveTabClass() {
        this.activeTag = "";
        this.openDrop = "";
        this.dropIndex = 0;
        this.dropSource.forEach(tag => tag.active = false);
    }
    subscribeFormControl() {
        this.currentFormControl.statusChanges.subscribe(t => {
            this.isInValid = t === "INVALID";
        });
    }
    validate(formControl) {
        if (!this.currentFormControl) {
            this.currentFormControl = formControl;
            this.subscribeFormControl();
        }
        return null;
    }
    ngOnDestroy() {
        let tString = '';
        this.selectedTags.forEach(t => tString += t.id);
        this.selectedTags = [];
        this.mainSource = [];
    }
};
RxTagComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
__decorate([
    ViewChild('tagSearch', { static: false })
], RxTagComponent.prototype, "viewChild", void 0);
__decorate([
    Output()
], RxTagComponent.prototype, "removed", void 0);
__decorate([
    Output()
], RxTagComponent.prototype, "added", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "mainClass", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "freeTextEnabled", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "lookup", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "controlTabIndex", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "formControl", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "isSort", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "params", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "queryParams", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "placeholder", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "source", null);
__decorate([
    Input()
], RxTagComponent.prototype, "tagDisabled", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "maxSelection", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "keyValueProps", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "focus", void 0);
__decorate([
    Input()
], RxTagComponent.prototype, "itemTexts", void 0);
RxTagComponent = RxTagComponent_1 = __decorate([
    Component({
        selector: 'rx-tag',
        template: "<div class=\"chosen-container chosen-container-multi  {{openDrop}} {{activeTag}} \" [class.rxMessage-control-element]=\"!isInValid\" style=\"width: 100%;\" title=\"\">\r\n    <ul class=\"chosen-choices form-control rounded-0\">\r\n        <li class=\"search-choice\" *ngFor=\"let tag of selectedTags\">\r\n            <span>{{tag.text}}</span>\r\n            <i class=\"close \" *ngIf=\"!tagDisabled\" (click)=\"removeTag(tag)\">&times;</i>\r\n        </li>\r\n        <li class=\"search-field\">\r\n            <input [tabindex]=\"controlTabIndex\" [class]=\"mainClass\" [placeholder]=\"selectedTags.length == 0 ? placeholder : ''\" type=\"text\" #tagSearch [disabled]=\"tagDisabled\" (click)=\"onFocus($event)\" (focus)=\"onFocus($event)\" (blur)=\"onBlur($event)\" (keydown)=\"onKeydown($event,scroller)\" (keydown.enter)=\"enter($event.target)\" (keyup)=\"onKeyup($event,scroller)\" value=\"\" class=\"\" autocomplete=\"off\">\r\n        </li>\r\n    </ul>\r\n    <div class=\"chosen-drop\" #scroller>\r\n        <ul class=\"chosen-results\" *ngIf=\"!isLimitExceeded\">\r\n            <li class=\"active-result\" *ngFor=\"let tag of dropSource \" (click)=\"onClick(tag,tagSearch)\" (mouseover)=\"onMouseover(tag)\" [class.highlighted]=\"tag.active\">{{tag.text}}</li>\r\n        </ul>\r\n        <ul class=\"chosen-results\" *ngIf=\"isLimitExceeded\">\r\n            <li class=\"active-result\">{{maxSelectionMessage}}</li>\r\n        </ul>\r\n    </div>\r\n</div>",
        providers: [
            { provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => RxTagComponent_1), multi: true },
            { provide: NG_VALIDATORS, useExisting: forwardRef(() => RxTagComponent_1), multi: true }
        ],
        exportAs: 'tag'
    })
], RxTagComponent);
let RxTagModule = class RxTagModule {
};
RxTagModule = __decorate([
    NgModule({
        imports: [CommonModule, FormsModule, ReactiveFormsModule],
        exports: [RxTagComponent],
        declarations: [RxTagComponent]
    })
], RxTagModule);

/**
 * Generated bundle index. Do not edit.
 */

export { RxTagComponent, RxTagModule };
//# sourceMappingURL=rxweb-angular-tag.js.map
